 MODULE diffusion_v_mod

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! Wall functions now include diabatic conditions, call of routine wall_fluxes
!
! Former revisions:
! -----------------
! $Id: diffusion_v.f90 53 2007-03-07 12:33:47Z raasch $
!
! 20 2007-02-26 00:12:32Z raasch
! Bugfix: ddzw dimensioned 1:nzt"+1"
!
! RCS Log replace by Id keyword, revision history cleaned up
!
! Revision 1.15  2006/02/23 10:36:00  raasch
! nzb_2d replaced by nzb_v_outer in horizontal diffusion and by nzb_v_inner
! or nzb_diff_v, respectively, in vertical diffusion,
! wall functions added for north and south walls, +z0 in argument list,
! terms containing w(k-1,..) are removed from the Prandtl-layer equation
! because they cause errors at the edges of topography
! WARNING: loops containing the MAX function are still not properly vectorized!
!
! Revision 1.1  1997/09/12 06:24:01  raasch
! Initial revision
!
!
! Description:
! ------------
! Diffusion term of the v-component
!------------------------------------------------------------------------------!

    PRIVATE
    PUBLIC diffusion_v

    INTERFACE diffusion_v
       MODULE PROCEDURE diffusion_v
       MODULE PROCEDURE diffusion_v_ij
    END INTERFACE diffusion_v

 CONTAINS


!------------------------------------------------------------------------------!
! Call for all grid points
!------------------------------------------------------------------------------!
    SUBROUTINE diffusion_v( ddzu, ddzw, km, km_damp_x, tend, u, v, vsws, w, z0 )

       USE control_parameters
       USE grid_variables
       USE indices

       IMPLICIT NONE

       INTEGER ::  i, j, k
       REAL    ::  kmxm_x, kmxm_y, kmxp_x, kmxp_y, kmzm, kmzp
       REAL    ::  ddzu(1:nzt+1), ddzw(1:nzt+1), km_damp_x(nxl-1:nxr+1)
       REAL    ::  z0(nys-1:nyn+1,nxl-1:nxr+1)
       REAL    ::  tend(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1)
       REAL, DIMENSION(nzb:nzt+1)      ::  vsus
       REAL, DIMENSION(:,:),   POINTER ::  vsws
       REAL, DIMENSION(:,:,:), POINTER ::  km, u, v, w

       DO  i = nxl, nxr
          DO  j = nys, nyn+vynp
!
!--          Compute horizontal diffusion
             DO  k = nzb_v_outer(j,i)+1, nzt
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmxp_x = 0.25 * &
                         ( km(k,j,i)+km(k,j,i+1)+km(k,j-1,i)+km(k,j-1,i+1) )
                kmxm_x = 0.25 * &
                         ( km(k,j,i)+km(k,j,i-1)+km(k,j-1,i)+km(k,j-1,i-1) )
                kmxp_y = kmxp_x
                kmxm_y = kmxm_x
!
!--             Increase diffusion at the outflow boundary in case of
!--             non-cyclic lateral boundaries. Damping is only needed for
!--             velocity components parallel to the outflow boundary in
!--             the direction normal to the outflow boundary.
                IF ( bc_lr /= 'cyclic' )  THEN
                   kmxp_x = MAX( kmxp_x, km_damp_x(i) )
                   kmxm_x = MAX( kmxm_x, km_damp_x(i) )
                ENDIF

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + ( kmxp_x * ( v(k,j,i+1) - v(k,j,i)     ) * ddx     &
                      &   + kmxp_y * ( u(k,j,i+1) - u(k,j-1,i+1) ) * ddy     &
                      &   - kmxm_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx         &
                      &   - kmxm_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy         &
                      &   ) * ddx                                            &
                      & + 2.0 * (                                            &
                      &           km(k,j,i)   * ( v(k,j+1,i) - v(k,j,i) )    &
                      &         - km(k,j-1,i) * ( v(k,j,i) - v(k,j-1,i) )    &
                      &         ) * ddy2
             ENDDO

!
!--          Wall functions at the left and right walls, respectively
             IF ( wall_v(j,i) /= 0.0 )  THEN

!
!--             Calculate the horizontal momentum flux v'u'
                CALL wall_fluxes( i, j, nzb_v_inner(j,i)+1, nzb_v_outer(j,i), &
                                  vsus, 0.0, 1.0, 0.0, 0.0 )

                DO  k = nzb_v_inner(j,i)+1, nzb_v_outer(j,i)
                   kmxp_x = 0.25 * &
                            ( km(k,j,i)+km(k,j,i+1)+km(k,j-1,i)+km(k,j-1,i+1) )
                   kmxm_x = 0.25 * &
                            ( km(k,j,i)+km(k,j,i-1)+km(k,j-1,i)+km(k,j-1,i-1) )
                   kmxp_y = kmxp_x
                   kmxm_y = kmxm_x
!
!--                Increase diffusion at the outflow boundary in case of
!--                non-cyclic lateral boundaries. Damping is only needed for
!--                velocity components parallel to the outflow boundary in
!--                the direction normal to the outflow boundary.
                   IF ( bc_lr /= 'cyclic' )  THEN
                      kmxp_x = MAX( kmxp_x, km_damp_x(i) )
                      kmxm_x = MAX( kmxm_x, km_damp_x(i) )
                   ENDIF

                   tend(k,j,i) = tend(k,j,i)                                   &
                                 + 2.0 * (                                     &
                                       km(k,j,i)   * ( v(k,j+1,i) - v(k,j,i) ) &
                                     - km(k,j-1,i) * ( v(k,j,i) - v(k,j-1,i) ) &
                                         ) * ddy2                              &
                                 + (   fxp(j,i) * (                            &
                                  kmxp_x * ( v(k,j,i+1) - v(k,j,i)     ) * ddx &
                                + kmxp_y * ( u(k,j,i+1) - u(k,j-1,i+1) ) * ddy &
                                                  )                            &
                                     - fxm(j,i) * (                            &
                                  kmxm_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx     &
                                + kmxm_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy     &
                                                  )                            &
                                     + wall_v(j,i) * vsus(k)                   &
                                   ) * ddx
                ENDDO
             ENDIF

!
!--          Compute vertical diffusion. In case of simulating a Prandtl
!--          layer, index k starts at nzb_v_inner+2.
             DO  k = nzb_diff_v(j,i), nzt
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmzp = 0.25 * &
                       ( km(k,j,i)+km(k+1,j,i)+km(k,j-1,i)+km(k+1,j-1,i) )
                kmzm = 0.25 * &
                       ( km(k,j,i)+km(k-1,j,i)+km(k,j-1,i)+km(k-1,j-1,i) )

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + ( kmzp * ( ( v(k+1,j,i) - v(k,j,i) ) * ddzu(k+1)   &
                      &            + ( w(k,j,i) - w(k,j-1,i) ) * ddy         &
                      &            )                                         &
                      &   - kmzm * ( ( v(k,j,i)   - v(k-1,j,i)   ) * ddzu(k) &
                      &            + ( w(k-1,j,i) - w(k-1,j-1,i) ) * ddy     &
                      &            )                                         &
                      &   ) * ddzw(k)
             ENDDO

!
!--          Vertical diffusion at the first grid point above the surface,
!--          if the momentum flux at the bottom is given by the Prandtl law
!--          or if it is prescribed by the user.
!--          Difference quotient of the momentum flux is not formed over
!--          half of the grid spacing (2.0*ddzw(k)) any more, since the
!--          comparison with other (LES) modell showed that the values of
!--          the momentum flux becomes too large in this case.
!--          The term containing w(k-1,..) (see above equation) is removed here
!--          because the vertical velocity is assumed to be zero at the surface.
             IF ( use_surface_fluxes )  THEN
                k = nzb_v_inner(j,i)+1
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmzp = 0.25 * &
                       ( km(k,j,i)+km(k+1,j,i)+km(k,j-1,i)+km(k+1,j-1,i) )
                kmzm = 0.25 * &
                       ( km(k,j,i)+km(k-1,j,i)+km(k,j-1,i)+km(k-1,j-1,i) )

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + ( kmzp * ( w(k,j,i) - w(k,j-1,i)     ) * ddy       &
                      &   ) * ddzw(k)                                        &
                      & + ( kmzp * ( v(k+1,j,i) - v(k,j,i) ) * ddzu(k+1)     &
                      &   + vsws(j,i)                                        &
                      &   ) * ddzw(k)
             ENDIF

          ENDDO
       ENDDO

    END SUBROUTINE diffusion_v


!------------------------------------------------------------------------------!
! Call for grid point i,j
!------------------------------------------------------------------------------!
    SUBROUTINE diffusion_v_ij( i, j, ddzu, ddzw, km, km_damp_x, tend, u, v, &
                               vsws, w, z0 )

       USE control_parameters
       USE grid_variables
       USE indices

       IMPLICIT NONE

       INTEGER ::  i, j, k
       REAL    ::  kmxm_x, kmxm_y, kmxp_x, kmxp_y, kmzm, kmzp
       REAL    ::  ddzu(1:nzt+1), ddzw(1:nzt+1), km_damp_x(nxl-1:nxr+1)
       REAL    ::  z0(nys-1:nyn+1,nxl-1:nxr+1)
       REAL    ::  tend(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1)
       REAL, DIMENSION(nzb:nzt+1)      ::  vsus
       REAL, DIMENSION(:,:),   POINTER ::  vsws
       REAL, DIMENSION(:,:,:), POINTER ::  km, u, v, w

!
!--    Compute horizontal diffusion
       DO  k = nzb_v_outer(j,i)+1, nzt
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmxp_x = 0.25 * ( km(k,j,i)+km(k,j,i+1)+km(k,j-1,i)+km(k,j-1,i+1) )
          kmxm_x = 0.25 * ( km(k,j,i)+km(k,j,i-1)+km(k,j-1,i)+km(k,j-1,i-1) )
          kmxp_y = kmxp_x
          kmxm_y = kmxm_x
!
!--       Increase diffusion at the outflow boundary in case of non-cyclic
!--       lateral boundaries. Damping is only needed for velocity components
!--       parallel to the outflow boundary in the direction normal to the
!--       outflow boundary.
          IF ( bc_lr /= 'cyclic' )  THEN
             kmxp_x = MAX( kmxp_x, km_damp_x(i) )
             kmxm_x = MAX( kmxm_x, km_damp_x(i) )
          ENDIF

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + ( kmxp_x * ( v(k,j,i+1) - v(k,j,i)     ) * ddx     &
                      &   + kmxp_y * ( u(k,j,i+1) - u(k,j-1,i+1) ) * ddy     &
                      &   - kmxm_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx         &
                      &   - kmxm_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy         &
                      &   ) * ddx                                            &
                      & + 2.0 * (                                            &
                      &           km(k,j,i)   * ( v(k,j+1,i) - v(k,j,i) )    &
                      &         - km(k,j-1,i) * ( v(k,j,i) - v(k,j-1,i) )    &
                      &         ) * ddy2
       ENDDO

!
!--    Wall functions at the left and right walls, respectively
       IF ( wall_v(j,i) /= 0.0 )  THEN

!
!--       Calculate the horizontal momentum flux v'u'
          CALL wall_fluxes( i, j, nzb_v_inner(j,i)+1, nzb_v_outer(j,i), &
                            vsus, 0.0, 1.0, 0.0, 0.0 )

          DO  k = nzb_v_inner(j,i)+1, nzb_v_outer(j,i)
             kmxp_x = 0.25 * &
                      ( km(k,j,i)+km(k,j,i+1)+km(k,j-1,i)+km(k,j-1,i+1) )
             kmxm_x = 0.25 * &
                      ( km(k,j,i)+km(k,j,i-1)+km(k,j-1,i)+km(k,j-1,i-1) )
             kmxp_y = kmxp_x
             kmxm_y = kmxm_x
!
!--          Increase diffusion at the outflow boundary in case of
!--          non-cyclic lateral boundaries. Damping is only needed for
!--          velocity components parallel to the outflow boundary in
!--          the direction normal to the outflow boundary.
             IF ( bc_lr /= 'cyclic' )  THEN
                kmxp_x = MAX( kmxp_x, km_damp_x(i) )
                kmxm_x = MAX( kmxm_x, km_damp_x(i) )
             ENDIF

             tend(k,j,i) = tend(k,j,i)                                         &
                                 + 2.0 * (                                     &
                                       km(k,j,i)   * ( v(k,j+1,i) - v(k,j,i) ) &
                                     - km(k,j-1,i) * ( v(k,j,i) - v(k,j-1,i) ) &
                                         ) * ddy2                              &
                                 + (   fxp(j,i) * (                            &
                                  kmxp_x * ( v(k,j,i+1) - v(k,j,i)     ) * ddx &
                                + kmxp_y * ( u(k,j,i+1) - u(k,j-1,i+1) ) * ddy &
                                                  )                            &
                                     - fxm(j,i) * (                            &
                                  kmxm_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx     &
                                + kmxm_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy     &
                                                  )                            &
                                     + wall_v(j,i) * vsus(k)                   &
                                   ) * ddx
          ENDDO
       ENDIF

!
!--    Compute vertical diffusion. In case of simulating a Prandtl layer,
!--    index k starts at nzb_v_inner+2.
       DO  k = nzb_diff_v(j,i), nzt
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmzp = 0.25 * ( km(k,j,i)+km(k+1,j,i)+km(k,j-1,i)+km(k+1,j-1,i) )
          kmzm = 0.25 * ( km(k,j,i)+km(k-1,j,i)+km(k,j-1,i)+km(k-1,j-1,i) )

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + ( kmzp * ( ( v(k+1,j,i) - v(k,j,i) ) * ddzu(k+1)   &
                      &            + ( w(k,j,i) - w(k,j-1,i) ) * ddy         &
                      &            )                                         &
                      &   - kmzm * ( ( v(k,j,i)   - v(k-1,j,i)   ) * ddzu(k) &
                      &            + ( w(k-1,j,i) - w(k-1,j-1,i) ) * ddy     &
                      &            )                                         &
                      &   ) * ddzw(k)
       ENDDO

!
!--    Vertical diffusion at the first grid point above the surface, if the
!--    momentum flux at the bottom is given by the Prandtl law or if it is
!--    prescribed by the user.
!--    Difference quotient of the momentum flux is not formed over half of
!--    the grid spacing (2.0*ddzw(k)) any more, since the comparison with 
!--    other (LES) modell showed that the values of the momentum flux becomes
!--    too large in this case.
!--    The term containing w(k-1,..) (see above equation) is removed here
!--    because the vertical velocity is assumed to be zero at the surface.
       IF ( use_surface_fluxes )  THEN
          k = nzb_v_inner(j,i)+1
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmzp = 0.25 * ( km(k,j,i)+km(k+1,j,i)+km(k,j-1,i)+km(k+1,j-1,i) )
          kmzm = 0.25 * ( km(k,j,i)+km(k-1,j,i)+km(k,j-1,i)+km(k-1,j-1,i) )

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + ( kmzp * ( w(k,j,i) - w(k,j-1,i)     ) * ddy       &
                      &   ) * ddzw(k)                                        &
                      & + ( kmzp * ( v(k+1,j,i) - v(k,j,i) ) * ddzu(k+1)     &
                      &   + vsws(j,i)                                        &
                      &   ) * ddzw(k)
       ENDIF

    END SUBROUTINE diffusion_v_ij

 END MODULE diffusion_v_mod
